#!/usr/bin/env python
import gtk.glade, gtk, gobject, os.path, time, os, sys, re, threading, gtk.gdk, Image, StringIO, pango, string
import prefs, shopgui, reccard
import rxml_to_metakit, rmetakit, convert, exporter, importer, rmmf_to_metakit, WidgetSaver
import dialog_extras as de
import treeview_extras as te
from get_pixbuf_from_file import get_pixbuf_from_jpg
from debug import *
from globals import *
from recindex import RecIndex
        
class RecGui (RecIndex):
    """This is the main application."""
    def __init__ (self,file=None,splash_label=gtk.Label()):
        self.splash_label = splash_label
        debug("__init__ (self,file=None):",5)
        self.update_splash("Loading window preferences...")
        self.prefs = prefs.Prefs()
        self.update_splash("Loading graphical interface...")        
        self.glade = gtk.glade.XML(os.path.join(gladebase,'app.glade'))
        self.pop = self.glade.get_widget('rlmen')
        self.app = self.glade.get_widget('app')
        self.conf = []
        self.conf.append(WidgetSaver.WindowSaver(self.app,
                                                 self.prefs.get('app_window',
                                                                {}),
                                                 show=False))
        self.prog = self.glade.get_widget('progressbar')
        self.selected = gtk.TRUE
        self.act_on_row_widgets = [self.glade.get_widget('rlViewRecButton'),
                                   self.glade.get_widget('rlViewRecMenu'),
                                   self.glade.get_widget('rlShopRecButton'),
                                   self.glade.get_widget('rlShopRecMenu'),
                                   self.glade.get_widget('rlDelRecButton'),
                                   self.glade.get_widget('rlDelRecMenu'),
                                   ]


        #self.file = {'db':'recs.db'}
        #self.file = {'xml':'recs.xml'}
        #self.file = {'mmf' : 'mealmaster.mmf'}
        if not file:
            directory=os.path.expanduser('~/.gourmet')
            if not os.path.isdir(directory):
                os.makedirs(directory)
            self.file={'db':os.path.expanduser('~/.gourmet/recipes.mk')}
        elif type(file)==type(""):
            self.file={self.guess_type(file):file}
        else:
            self.file=file
        self.rtcolsdic={}
        for a,l,w in REC_ATTRS:
            self.rtcolsdic[a]=l
        self.rtcols=map(lambda r: r[0], REC_ATTRS)
        self.update_splash("Loading recipe database...")
        self.init_recipes(**self.file)
        self.update_splash("Setting up recipe index...")
        RecIndex.__init__(self, model=self.rmodel, glade=self.glade, rd=self.rd, rg=self)
        self.rectree.connect("popup-menu",self.popup_rmenu)#self.recTreeSelectRec)
        self.rectree.connect("button-press-event",self.rectree_click_cb)
        self.glade.signal_autoconnect({
            'newRec' : self.newRecCard,
            'shopHide' : self.sl.hide,
            'showShop' : self.sl.show,
            'showList' : lambda *args: self.app.present(),            
            'new' : self.new,
            'open': self.open,
            'defaultsave': self.save_default,
            'export_xml': self.exportXMLg,
            'export_text': self.exportTextg,
            'export_html': self.exportHTMLg,
            'export_mealmaster': self.exportMMFg,
            'importXML' : self.importXMLg,
            'importMMF' : self.importMMF,
            'quit' : self.quit,
            'about' : self.show_about,
            'rl_viewrec' : self.recTreeSelectRec,
            'rl_shoprec' : self.recTreeShopRec,
            'rl_delrec': self.recTreeDeleteRec,
            'colPrefs': self.configureColDialog,
            })
        self.rc={}
        self.updateViewMenu()
        ## make sure the focus is where it ought to be...
        self.app.present()
        self.srchentry.grab_focus()
        self.update_splash("Done!")
        # used on imports to make filtering
        # wait until we are all done.
        self.wait_to_filter = False

    def update_splash (self, text):
        """Update splash screen on startup."""
        debug("Setting splash text: %s"%text,3)
        self.splash_label.set_text(text)
        while gtk.events_pending():
            gtk.main_iteration()
        
    def del_rc (self, id):
        """Delete recipe with ID=id"""
        if self.rc.has_key(id):
            del self.rc[id]
        self.updateViewMenu()
        
    def viewMenu (self):
        """Build a _View menu based on recipes currently
        opened in recipe cards."""
        m=gtk.Menu()
        ri=gtk.MenuItem('Recipe _Index')
        sh=gtk.MenuItem('Shopping _List')
        separator=gtk.MenuItem()
        ri.connect('activate',lambda *args: self.app.present())
        sh.connect('activate',self.sl.show)
        m.append(ri)
        ri.show()
        m.append(sh)
        sh.show()
        m.append(separator)
        separator.show()
        for rc in self.rc.values():
            i=gtk.MenuItem("_%s"%rc.current_rec.title)
            i.connect('activate',rc.show)
            m.append(i)
            i.show()
        return m

    def updateViewMenu (self):
        """Update view menus in all open windows."""
        glades=[self.glade, self.sl.glade]
        for r in self.rc.values():
            glades.append(r.glade)
        for glade in glades:
            menu=self.viewMenu()
            vmi=glade.get_widget('view_menu_item')
            vmi.set_submenu(menu)
    
    def show_about (self, *args):
        """Show information about ourselves, using GNOME's
        nice interface if available."""
        debug("show_about (self, *args):",5)
        description="Gourmet is a Recipe Manager for GNOME"
        copyright="Copyright (c) 2004, Thomas M. Hinkle. GNU GPL"
        appname="Gourmet Recipe Manager"
        version="0.5.2"
        authors=["Thomas M. Hinkle <Thomas_Hinkle@alumni.brown.edu"]
        documenters=None
        translators=None
        comments=None
        logo=gtk.gdk.pixbuf_new_from_file(os.path.join(imagedir,"gourmet_logo.png"))
        #try:
        if 1:
            import gnome.ui
            about= gnome.ui.About(appname,
                                  version,
                                  copyright,
                                  description,
                                  authors,
                                  comments,
                                  translators,
                                  logo)
            about.show()
        #except:
        #    de.show_message(label="%s %s\n%s\n%s\n%s"%(appname,version,copyright,description,authors))
          
    def quit (self, *args):
        """Close down shop, giving user option of saving changes and
        saving our window prefs for posterity."""
        debug("quit (self, *args):",5)
        for c in self.conf:
            c.save_properties()
        a=self.glade.get_widget('app')
        saved= not self.rd.changed
        saveMe=False
        for r in self.rc.values():
            for c in r.conf:
                c.save_properties()
            if r.edited and de.getBoolean(parent=self.app,
                                             label="Save your edits to %s"%r.current_rec.title):
                r.saveEditsCB()
                saveMe=True
            else: r.edited=False # in case someone else checks this (e.g. reccard on close)
        for conf in self.sl.conf:
            conf.save_properties()
        self.prefs.save()
        if saveMe:
            self.save_default()
        elif not saved and de.getBoolean(label="Save database before quitting?", parent=self.app.get_toplevel()):
            self.save_default()
        for r in self.rc.values():
            r.widget.destroy()
        self.sl.widget.destroy()
        a.destroy()
        gtk.main_quit()
        
    def new (self, *args):
        debug("new (self, *args):",5)
        self.init_recipes()

    def guess_type (self, filestr):
        """Handed a file representing recipes,
        guess which type it is."""
        debug("guess_type (self, filestr):",5)
        if re.search("\.(db|mk)$",filestr):
            return "db"
        elif re.search("\.xml$",filestr):
            return "xml"
        elif re.search("\.mm.$",filestr):
            return "mmf"
        else: return "db"
        
    def open (self, *args):
        debug("open (self, *args):",5)
        f = de.select_file()
        debug("opening %s"%f, 1)
        if f:
            t = self.guess_type(f)
            ## we should probably do better than looking at extensions
            ## eventually **FIX ME**
            if t=="db":
                self.init_recipes(db=f)
            elif t=="xml":
                self.init_recipes(xml=f)
            elif t=="mmf":
                self.init_recipes(mmf=f)
            else:
                self.message("I can't determine the type of file %s"%f)

    def init_recipes (self, xml=None, db=None, mmf=None, defaultxml='recs.xml'):
        debug("init_recipes (self, xml=None, db=None, mmf=None, defaultxml='recs.xml'):        ",5)
        if xml:
            self.xtm = rxml_to_metakit.converter(xml)
            self.rd = self.xtm.db
        elif db:
            self.rd = rmetakit.recipeManager(db)
        elif mmf:
            self.rd = rmetakit.recipeManager()
            rmmf_to_metakit.mmf_importer(self.rd, filename=mmf)
        else:
            raise "No file specified"
        # connect hooks to modify our view whenever and
        # whenceever our recipes are updated...
        self.rd.modify_hooks.append(self.update_rec_iter)
        self.rd.add_hooks.append(self.new_rec_iter)
        self.rd.delete_hooks.append(self.delete_rec_iter)
        # a flag to make deleting multiple recs
        # more efficient...
        self.doing_multiple_deletions=False
        self.conv = convert.converter()
        self.create_rmodel(self.rd.rview)        
        self.sl = shopgui.ShopGui(self)
        self.sl.hide()

    def selection_changed (self, selected=gtk.FALSE):
        if selected != self.selected:
            if selected: self.selected=gtk.TRUE
            else: self.selected=gtk.FALSE
            for w in self.act_on_row_widgets:
                w.set_sensitive(self.selected)

    def rectree_click_cb (self, tv, event):
        debug("rectree_click_cb (self, tv, event):",5)
        if event.button==3:
            self.popup_rmenu()
            return gtk.TRUE

    def reset_rtree (self):
        debug("reset_rtree (self):",5)
        #self.rmodel=self.create_rmodel(self.rd.rview)
        #self.rectree.set_model(self.rmodel)
        self.update_rmodel(self.rd.rview)
        self.search()
        self.selection_changed()
        self.set_reccount()
        #for r in rview:
        #    if not r.id in ids:
        #        # if we're already there...
        #        iter=self.rmodel.append(None)
        #        self.set_iter_from_rec(r,iter)

    def new_rec_iter (self, rec):
        debug("new_rec_iter called",5)
        iter = self.rmodel.append(None)
        self.set_iter_from_rec(rec,iter)
        

    def delete_rec_iter (self, rec):
        # a simpler, but less correct solution
        # is to simply make deleted item invisible.
        #self.visible.remove(rec.id)
        if self.doing_multiple_deletions: return
        for row in self.rmodel:
            if row[0].id==rec.id:
                debug("delete_rec_iter found the row to delete",5)
                dbg = ""
                for r in row:
                    dbg += str(r)
                    dbg += " | "
                debug("delete_rec_iter acting on: %s"%dbg,5)
                # this seems to be necessary to prevent
                # a segfault on removing the last iter
                self.rectree.set_model(None)
                row.model.remove(row.iter)
                self.rectree.set_model(self.rmodel_sortable)
                debug('delete_rec_iter removed recipe with ID %s'%rec.id,2)

        debug('delete_rec_iter couldn\'t find recipe with ID %s'%rec.id,1)

    def update_rec_iter (self, rec):
        # if r is already in our treeModel, we update values.
        # otherwise, we add it.
        debug("update_rec_iter called",5)
        for row in self.rmodel:
            recobj = row[0]
            if recobj.id == rec.id:
                row[0]=rec
                if rec.thumb:
                    row[1]=get_pixbuf_from_jpg(rec.thumb)
                else:
                    row[1]=None                
                n = 2
                for c in self.rtcols:
                    row[n]=str(getattr(rec,c))
                    n+=1
                debug("update_rec_iter done!, id=%s"%rec.id,)
                return
        else: self.new_rec_iter(rec)
    
    def set_iter_from_rec (self, r, iter, visible=gtk.TRUE):
        """Handed a rec and an iter, set the row values appropriately."""
        self.rmodel.set_value(iter, 0, r)
        if r.thumb:
            self.rmodel.set_value(iter, 1, get_pixbuf_from_jpg(r.thumb))
        else:
            self.rmodel.set_value(iter, 1, None)
        n = 2
        for c in self.rtcols:
            debug("setting column %s (n=%s)"%(c,n),5)
            self.rmodel.set_value(iter, n, str(getattr(r,c)))
            n += 1

    def create_rmodel (self, rview):
        debug("create_rmodel (self, rview):",5)
        # we start with our recipe object and our image
        mod = [gobject.TYPE_PYOBJECT,gtk.gdk.Pixbuf]
        # we add columns for all attributes...
        for n in self.rtcols:
            mod.append(gobject.TYPE_STRING)
        self.rmodel = apply(gtk.TreeStore,mod)
        # now we add our recipes
        for r in rview:
            self.new_rec_iter(r)

    def recTreeDeleteRecCB (self, *args):
        """Make a watch show up (this can be slow
        if lots of recs are selected!"""
        gtk.app.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        gtk.idle_add(self.recTreeDeleteRec)

    def recTreeDeleteRec (self, *args):
        debug("recTreeDeleteRec (self, *args):",5)
        mod,rr=self.rectree.get_selection().get_selected_rows()
        #recs = map(lambda path: self.get_rec_from_iter(mod.get_iter(path)), rr)
        recs = map(lambda path: mod[path][0], rr)
        expander=None
        if len(recs) == 1:
            msg = "Are you sure you want to delete the recipe <i>%s</i>"%recs[0].title
        elif len(recs) < 5:
            msg = "Are you sure you want to delete the following recipes?"
            for r in recs:
                msg += "\n<i>%s</i>"%r.title
        else:
            msg = "Are you sure you want to delete the %s selected recipes?"%len(recs)
            tree = te.QuickTree(map(lambda r: r.title, recs))
            expander = ["See recipes",tree]
        if de.getBoolean(parent=self.app,label=msg,expander=expander):
            self.doing_multiple_deletions=True
            self.iters_to_remove=[]
            for p in rr:
                # we remove by hand...
                iter = self.rmodel_sortable.get_iter(p)
                child = self.rmodel_sortable.convert_iter_to_child_iter(None, iter)
                grandchild = self.rmodel_filter.convert_iter_to_child_iter(child)
                self.iters_to_remove.append(grandchild)
            self.iters_to_remove.sort()
            self.iters_to_remove.reverse()
            self.rectree.set_model(None)
            for i in self.iters_to_remove:
                self.rmodel.remove(i)
            self.rectree.set_model(self.rmodel_sortable)
            def do_deletions (*args):
                """We separate out this to call
                in a separate thread (for big deletes)"""
                rtot=len(recs)
                n = 0
                for r in recs:
                    self.set_progress_thr(float(n)/float(rtot),
                                          "Total Recipes (Deleting recipes from database...)")
                    self.delete_rec(r)
                    n += 1
                self.reset_prog_thr()
                gtk.threads_enter()
                self.message("Deleted %s recipes."%rtot)
                self.doing_multiple_deletions=False
                gtk.threads_leave()

            gtk.threads_leave()
            t=threading.Thread(target=do_deletions,name='delete')
            t.start()
            
        self.app.window.set_cursor(None)
            
    def delete_rec (self, rec):
        debug("delete_rec (self, rec): %s"%rec,5)
        debug("does %s have %s"%(self.rc,rec.id),5)
        if self.rc.has_key(rec.id):
            debug("Getting rid of open recipe card window.",2)
            w=self.rc[rec.id].widget
            self.rc[rec.id].hide()
            w.destroy()
            self.updateViewMenu()
        if hasattr(rec,'id') and rec.id:
            titl = rec.title
            debug('deleting recipe',5)
            self.rd.delete_rec(rec.id)
            if not self.doing_multiple_deletions:
                self.message("Deleted recipe %s"%titl)
        else:
            debug("%s %s does not have an ID!"%(rec,rec.title))
        return None    

    def popup_rmenu (self, *args):
        debug("popup_rmenu (self, *args):",5)
        self.pop.popup(None,None,None,0,0)

    def newRecCard (self, *args):
        self.app.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        def show ():
            rc=reccard.RecCard(self)
            self.rc[rc.current_rec.id]=rc
            self.app.window.set_cursor(None)
        gtk.idle_add(show)

    def openRecCard (self, rec):
        if self.rc.has_key(rec.id):
            self.rc[rec.id].show()
        else:
            def show ():
                w=reccard.RecCard(self, rec)
                self.rc[rec.id]=w
                self.updateViewMenu()
                self.app.window.set_cursor(None)
            self.app.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
            gtk.idle_add(show)
    def recTreeSelectRec (self, *args):
        debug("recTreeSelectRec (self, *args):",5)
        for rec in self.recTreeSelectedRecs():
            self.openRecCard(rec)

    def recTreeShopRec (self, *args):
        debug("recTreeShopRec (self, *args):",5)
        rr=self.recTreeSelectedRecs()
        #r = self.recTreeSelectedRec()
        for r in rr:
            if r.servings and r.servings != "None":
                debug("servings=%s"%r.servings,0)
                serv = de.getNumber(default=float(r.servings),
                                    label="Number of Servings of %s to Shop For"%r.title,
                                    parent=self.app.get_toplevel())
                if serv: mult = float(serv)/float(r.servings)
                else:
                    debug('getNumber cancelled',2)
                    return
            else:
                mult = de.getNumber(default=float(1),
                                    label="Multiply %s by: "%r.title,
                                    parent=self.app.get_toplevel(),
                                    digits=2)
                if not mult:
                    mult = float(1)
            d=shopgui.getOptionalIngDic(self.rd.get_ings(r),mult)
            self.sl.addRec(r,mult,d)
            self.sl.show()

    def saveg (self, *args):
        debug("saveg (self, *args):",5)
        self.save(de.select_file())

    def save_default (self, *args):
        debug("save_default (self, *args):",5)
        self.save(**self.file)

    def save (self, file=None, db=None, xml=None):
        debug("save (self, file=None, db=None, xml=None):",5)
        if not xml and not db:
            if re.search(".xml$",file):
                xml=file
            else:
                db=file
        if xml:
            self.exportXML(file)
        else:
            self.rd.file=db
            self.rd.save()
            self.message("Saved!")

    def message (self, msg):
        debug("message (self, msg): %s"%msg,5)
        self.stat.push(self.contid,msg)
        gtk.timeout_add(1500,self.flush_messages)

    def flush_messages (self):
        debug("flush_messages (self):",5)
        self.stat.pop(self.contid)
        return False

    def exportMMFg (self, *args):
        debug("exportMMFg (self, *args):",5)
        file=de.select_file("Export Mealmaster (tm) File",filename="~/recipes.mmf")
        if file:
            gtk.threads_leave()
            t=threading.Thread(target=lambda *args: self.exportMMF(file), name='export')
            t.start()
            
        else:
            de.show_message(label="Mealmaster export cancelled!")
            
    def exportMMF (self, file):
        debug("exportMMF (self, file):",5)
        prog = lambda prog: self.set_progress_thr(prog, "total recipes. %i%% done exporting mealmaster file."%(prog*100))
        exporter.ExporterMultirec(self.rd, self.rd.rview,
                         file, one_file=True, ext='mmf',
                         conv=self.conv, progress_func=prog,
                         exporter=exporter.mealmaster_exporter)
        self.reset_prog_thr()

    def exportHTMLg (self, *args):
        debug("exportHTMLg (self, *args):",5)
        file=de.select_file("Export Webpage to Directory...",filename="~/my_recipes",file_required=False)
        if file:
            gtk.threads_leave()
            t=threading.Thread(target=lambda *args: self.exportHTML(file), name='export')
            t.start()
        else:
            de.show_message(label="Webpage export cancelled!")            
    def exportHTML (self, file):
        prog = lambda prog: self.set_progress_thr(prog, "total recipes. %i%% done creating web page."%(prog*100))        
        exporter.website_exporter(self.rd,self.rd.rview,
                                  file,self.conv, progress_func=prog)
        self.reset_prog_thr()
        
    def exportTextg (self, *args):
        debug("exportTextg (self, *args):",5)
        file=de.select_file("Export Plain Text File",filename="~/recipes.txt")
        if file:
            gtk.threads_leave()
            t=threading.Thread(target=lambda *args: self.exportText(file), name='export')
            t.start()        
        else:
            de.show_message(label="Text export cancelled!")

    def exportText (self, file):
        debug("exportText (self, file):",5)
        prog=lambda prog: self.set_progress_thr(prog, "total recipes. %i%% done exporting text."%(prog*100))
        exporter.ExporterMultirec(self.rd,self.rd.rview,file,
                                  conv=self.conv, progress_func=prog)
        self.reset_prog_thr()

    def exportXMLg (self, *args):
        debug("exportXMLg (self, *args):",5)
        file=de.select_file("Export XML",filename="~/recipes.xml")
        if file:
            gtk.threads_leave()
            t=threading.Thread(target=lambda *args: self.exportXML(file), name='export')
            t.start()
        else:
            de.show_message(label="XML export cancelled.")
            
    def exportXML (self,file):
        debug("exportXML (self,file):",5)
        """Export XML file"""
        prog = lambda prog: self.set_progress_thr(prog, "total recipes. %i%% done exporting XML file."%(prog*100))        
        rxml_to_metakit.rview_to_xml(self.rd, self.rd.rview, file, progress_func=prog)
        self.reset_prog_thr()

    def importXMLg (self, *args):
        debug("importXMLg (self, *args):",5)
        file=de.select_file("Import XML")
        self.app.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        if file:
            self.wait_to_filter=True
            def do_import ():
                try:
                    self.importXML(file)
                    self.refilter()
                    self.app.window.set_cursor(None)
                except:
                    self.app.window.set_cursor(None)            
                    de.show_message(label="There was an error which prevented Gourmet from importing %s"%file)
            gtk.idle_add(do_import)

    def importXML (self, file):
        debug("importXML (self, file):",5)
        c = rxml_to_metakit.converter(file,self.rd)
        
    def importMMF (self, file):
        debug("importMMF (self, file):",5)
        file=de.select_file("Import Mealmaster")
        if file:
            source=de.getEntry(label="Default source for memories imported from %s"%file,
                               default=os.path.split(file)[1], parent=self.app)
        if file and source:
            self.wait_to_filter=True
            gtk.threads_leave()
            t=threading.Thread(target=lambda *args: self.do_importMMF(file,source),name="import")
            t.start()
    
    def do_importMMF (self, file,source):
        debug("do_importMMF (self, file,source):",5)
        #file=de.select_file("Import Mealmaster")
        #gtk.threads_enter()
        rmmf_to_metakit.mmf_importer(filename=file, rd=self.rd, progress=self.set_progress_thr,source=source)
        #gtk.threads_leave()
        self.reset_prog_thr()
        self.refilter()        

    def refilter (self, from_thread=True):
        debug("refiltering",0)
        if from_thread: gtk.threads_enter()
        self.rmodel_filter.refilter()
        self.wait_to_filter=False
        if from_thread: gtk.threads_leave()

    def reset_prog_thr (self):
        gtk.threads_enter()
        self.prog.set_percentage(0)
        self.set_reccount()
        gtk.threads_leave()
        
    def set_progress_thr (self, prog, message="Total Recipes (Importing...)"):
        debug("set_progress_thr (self, prog,msg): %s"%prog,5)
        gtk.threads_enter()
        self.prog.set_percentage(prog)
        self.stat.push(self.contid,"%s %s"%(len(self.rd.rview),message))
        gtk.threads_leave()

    def configureColDialog (self, *args):
        already_hidden=self.prefs.get('rectree_hidden_columns',None)
        def mapper (i):
            if i in already_hidden: return [i, False]
            else: return [i, True]
        options=map(lambda i: self.rtcolsdic[i], self.rtcols)
        options=map(mapper, options)
        pd = de.preferences_dialog(options=options, option_label=None, value_label="Show in Index View",
                                   apply_func=self.configure_columns, parent=self.app)
        pd.show()

    def configure_columns (self, retcolumns):
        hidden=[]
        for c,v in retcolumns:
            if not v: hidden.append(c)
        self.rectree_conf.hidden=self.prefs['rectree_hidden_columns']=hidden
        self.rectree_conf.apply_visibility()

def startGUI ():
    debug("startGUI ():",4)
    # show splash screen before we do anything...
    debug("showing splash screen...",1)
    gtk.threads_init()
    gtk.threads_enter()
    splash = gtk.Window()
    #splash.window_set_auto_startup_notification(gtk.FALSE)
    splash.set_property('decorated',gtk.FALSE)
    splash.set_position(gtk.WIN_POS_CENTER)
    splash.set_icon_from_file(os.path.join(imagedir,'recbox.png'))
    splash.set_title('Gourmet Recipe Manager starting up...')
    pixbuf=gtk.gdk.pixbuf_new_from_file(os.path.join(imagedir,'splash.png'))
    pixmap, mask = pixbuf.render_pixmap_and_mask()
    width, height = pixmap.get_size()
    del pixbuf
    splash.set_app_paintable(gtk.TRUE)
    splash.resize(width, height)
    splash.realize()
    splash.window.set_back_pixmap(pixmap, gtk.FALSE)
    splash.label = gtk.Label("Starting gourmet...")
    splash.label.set_alignment(0.5,1)
    splash.label.set_justify(gtk.JUSTIFY_CENTER)
    splash.label.show()
    splash.add(splash.label)
    del pixmap
    splash.show()
    splash.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
    while gtk.events_pending():
        # show our GUI
        gtk.main_iteration()
    try:
        r=RecGui(splash_label=splash.label)
    except:
        splash.hide()
        while gtk.events_pending():
            gtk.main_iteration()
        #gtk.main_quit()
        raise
    debug('hiding splash screen.',1)
    splash.hide()
    gtk.main()
    gtk.threads_leave()
              
if __name__ == '__main__':
    startGUI()
    
