import re

class converter:
    def __init__(self):
        self.conv_table = {
            ("c.", "tbs."):16,
            ("lb.", "oz."):16,
            ("tbs.", "tsp."):3,
            ("pt.", "c."):2,
            ("qt.", "c."):4,
            ("gallon", "qt."):4,
            ("l.", "qt."):1.057,
            ("l.", "ml."):1000,
            ("l.", "cl."):100,
            ("l.", "dl."):10,
            ("oz.", "g."):28.35,
            ("kg.", "g."):1000,
            ("g.", "mg."):1000,
            ("tsp.", "drop"):76,
            ("oz.", "dram"):16,
            ("dram", "grains"):27.34375,
            ("peck", "gallon"):2,
            ("bucket", "peck"):2,
            ("bushel", "bucket"):2,
            ("lb.", "grains"):7000}

        self.density_table = {
            "water":1,
            "vegetable broth":1,
            "broth, vegetable":1,
            "broth, chicken":1,
            "milk":1,
            "milk, whole":1,
            "milk, skim":1,
            "milk, 2%":1,
            "milk, 1%":1,
            "coconut milk":0.875,
            "buttermilk":1,
            "heavy cream":1,
            "half and half":1,
            "honey":1.5,
            "sugar":1,
            "sugar, brown":1,
            "sugar, white":1,
            "salt":1,
            "butter":1,
            "oil, vegetable":1,
            "oil, olive":1,
            "oil, corn":1,
            "oil, sesame":1,
            "flour, all purpose": 0.6,
            "flour, whole wheat": 0.6,
            "cornstarch": 0.6,
            "sugar, powdered": 0.6,
            "sugar, confectioners": 0.6
            }
        
        self.cross_unit_table = {
            ## This if for units that require an additional
            ## bit of information -- i.e. to convert between
            ## volume and mass you need the density of an
            ## item.  In these cases, the additional factor
            ## will be provided as an 'item' that is then looked
            ## up in the dictionary referenced here (i.e. the density_table)
            ("pt.", "lb."):[self.density_table,1],
            ("tbs.", "oz."):[self.density_table,2],
            ("c.", "oz."):[self.density_table,8],
            ("cl.", "g."):[self.density_table,1]}
        
        ## This allows for varied spellings of units to be entered.
        self.units = [["bucket" , ["bucket", "buckets", "bckt."]],
                      ["peck", ["peck", "pecks"]],
                      ["bushel", ["bushel", "bushels", "bsh.", "bu.", "bu", "bsh", "bshl", "bshl."]],
                      ["grains", ["grain", "grains"]],
                      ["dram", ["dram", "drams"]],
                      ["ml.", ["ml.", "mL", "mL.", "ml"]],
                      ["tsp.", ["tsp", "tsp.", "teaspoon", "tea spoon", "tsps.", "teaspoons", "tea spoons", "Teaspoon", "t"]],
                      ["tbs.", ["tbs", "tbsp", "tbs.", "tbsp.", "tablespoon", "table spoon", "tbsps.", "tablespoons", "Tablespoon", "T"]],
                      ["lb.", ["lb", "pound", "lb.", "lbs.", "pounds"]],
                      ["oz.", ["oz", "ounce", "ounces", "oz."]],
                      ["c.", ["cup", "c.", "cups"]],
                      ["qt.", ["quart", "qt.", "quarts"]],
                      ["pt.", ["pint", "pt.", "pints"]],
                      ["gallon", ["gallon", "gallons", "gal."]],
                      ["ml.", ["ml", "ml.", "mililiter", "mililiters"]],
                      ["cl.", ["cl", "cl.", "centiliter", "centiliters"]],
                      ["dl.", ["dl", "dl.", "deciliter", "deciliters"]],
                      ["l.", ["liter", "l.", "lit.", "liters"]],
                      ["g.", ["grams", "gram", "g."]],
                      ["mg.", ["mg.", "mg", "miligram", "miligrams"]],
                      ["kg.", ["kg.", "kg", "kilogram", "kilograms"]]]
        self.create_unit_dict()
    def create_unit_dict(self):
        self.unit_dict={}
        for itm in self.units:
#            print itm
            key = itm[0]
#            print "key", key
            variations = itm[1]
#            print "vars", variations
            self.unit_dict[key] = key
            for v in variations:
                self.unit_dict[v] = key

    def convert_simple (self, u1, u2, dict=0, dictcu=None, item=None):
        if u1 == u2:
            return 1.0
        else:
            if (not dict):
                dict=self.conv_table
            if (not dictcu):
                dictcu=self.cross_unit_table
            if dict.has_key((u1,u2)):
                return dict[(u1,u2)]
            elif dict.has_key((u2,u1)):
                return float(1) / float(dict[(u2,u1)])
            else:
                return 0

    def list_of_cu_tables (self, dictcu=None):
        if (not dictcu):
            dictcu = self.cross_unit_table
        values = dictcu.values()
        ret = []
        for v in values:
            if not v[0] in ret:
                ret.append(v[0])
        return ret
                

    def conv_dict_for_item (self, item, dictcu=None):
        if (not dictcu):
            dictcu = self.cross_unit_table
#        tbls = self.list_of_cu_tables(dictcu)
#        my_tbls = []
        ret = {}
        for itm in dictcu.items():
            k = itm[0]
            v = itm[1]
            dct = v[0]
            conv = v[1]
            if dct.has_key(item):
                mult = dct[item]
                ret[k] = conv * mult
#        print 'special dict for', item,':',ret
        return ret

    def converter (self, u1, u2, item=None):
        ## Just a front end to convert_fancy that looks up units
        if self.unit_dict.has_key(u1):
            unit1 = self.unit_dict[u1]
        else:
            unit1 = u1
        if self.unit_dict.has_key(u2):
            unit2 = self.unit_dict[u2]
        else:
            unit2 = u2
        ## provide some kind of item lookup?
        return self.convert_fancy(unit1, unit2, item)

    def convert_fancy (self, u1, u2, item=None):
        cdict = self.conv_table
        if item:
            cdict = cdict.copy()
            cdict.update(self.conv_dict_for_item(item))
        simple = self.convert_simple(u1,u2, cdict)
        if simple:
            return simple
        else:
#            print "here we go..."
            tried = []
            dict1 = self.possible_conversions(u1, dict=cdict)
            dict2 = self.possible_conversions(u2, dict=cdict)
            ## We're going to move from dict1 and see if we can't find something that
            ## will hit on dict 2.  For simplicity's sake, we'll add u2 to its own dict
            ## with a conversion factor of 1
            dict2[u2] = 1
            dict1[u1] = 1
            lst = dict1.keys()
#            print 'dict 1:', dict1
#            print 'dict 2:', dict2
#            print 'Starting with', u1, 'and looking for',u2
            while len(lst) >= 1:
#                print 'Tried...', tried
#                print "Yet to try...", lst ##DEBUG
                itm = lst.pop()
#                print "Trying", itm
                if dict2.has_key(itm):
                    ## A hit!
#                    print "A hit! A hit!" ##DEBUG
#                    print 'dict1',dict1   ##DEBUG
#                    print 'dict2',dict2   ##DEBUG
                    c1 = dict1[itm]
                    c2 = dict2[itm]
#                    print u1, 'to', itm, ':', c1
#                    print '/'
#                    print u2, 'to', itm, ':', c2
                    return c2 / c1
                else:
#                    print 'no luck'
                    ## Otherwise, we add ourselves the the tried list
                    tried.append(itm)
                    ## and see what we can convert to ourselves...
                    newdict = self.possible_conversions(itm, dict=cdict)
#                    print 'new possibilities...', newdict
                    for new_key in newdict.keys():
                        if (not new_key in tried):
                            ## If we haven't been tried already... add ourselves to the dict
                            ## with the appropriate conversion factor multiplied in...
                            newval = newdict[new_key] * dict1[itm]
#                            print itm, dict1[itm], '*', new_key, newdict[new_key], '=', newval                            
                            dict1[new_key] = newval
                            lst.append(new_key)
                            ## And while we're add it, we'll add ourselves to the general
                            ## dictionary to speed things up in the future.
                            ## No, we won't do this any more since we may mix up
                            ## our fancy crossover units
                            #self.conv_table[(u1, new_key)] = newval
#                            print 'new key and val', new_key, ':', newval

		## Anything on the tried lst must be taken off
                ## our lst 
                for i in tried:
                    while i in lst:
                        lst.remove(i)

                    
    def possible_conversions(self, u, dict=0):
        """Return a dictionary of everything that unit u can convert to
        The keys are what it can convert to and the values are the conversion
        factor."""
        if (not dict):
            dict=self.conv_table
        ret = {}
        entries = dict.items()
        for item in entries:
            i1 = item[0][0]
            i2 = item[0][1]
#            print 'i1 =', i1, 'i2=', i2
            if u == i1:
                ret[i2] = float(1) / item[1]
            elif u == i2:
                ret[i1] = float(item[1])
        return ret            

    def readability_score (self,amt):
        """We rate the readability of a number"""
        readability = 0
        #print 'readability:',readability

        ## We like our numbers whole or in normal fractions
        if integerp(amt):
            #print 'integer'
            readability += 2
            #print 'readability', readability
        elif integerp(amt * 2):
            #print '1/2'                
            readability += 1.5
            #print 'readability', readability
        elif integerp(amt * 4):
            #print '1/4'                                
            readability += 1.25
            #print 'readability', readability
        elif integerp(amt * 3):
            #print '1/3'
            readability += 1.00
            #print 'readability', readability
        elif integerp (amt * 8):
            #print '1/8'
            readability += 0.8
            #print 'readability', readability
        elif integerp (amt * 6):
            #print '1/6'
            readability += 0.3
            #print 'readability', readability
        elif integerp (amt * 5):
            #print '1/5'
            readability += 0.2
            #print 'readability', readability
        elif integerp (amt * 10):
            #print '1/10'
            readability += 0.05
            #print 'readability', readability
        ## If it is not a normal fraction, readability takes a hit
        else:
            #print 'Not any kind of number I know...'
            readability += -1
            #print 'readability', readability
        ## And we like numbers between 1/8 and 4
        if 0.25 <= amt <= 4:
            #print 'ideal range'
            readability += 1
            #print 'readability', readability
        ## Less than 1/10th is getting too small
        elif 0.1 >= amt:
            #print 'less than a tenth'
            readability += -1
            #print 'readability', readability
        ## And we rather dislike numbers over 10
        elif 20 >= amt > 10:
            #print 'over 10' ## DEBUG
            readability += -0.9
            #print 'readability', readability
        ## And we really can't have numbers over 20
        elif 100 >= amt > 20:
            #print 'over 20' ##DEBUG
            readability += -5
            #print 'readability', readability
        ## And we really, really, really can't have numbers over 100
        elif 1000 >= amt > 100:
            readability += -10
        elif amt >= 1000:
            readability += -20
        return readability

    def use_reasonable_unit (self, amt1, u1, amt2, u2, conv):
        """Given the conversion factor and the amounts,
        we're going to try to figure out which unit
        is the most human-readable.  conv is what converts
        from amt1 into amt2.  We return a list of
        amt unit, where the amount is our total, and the unit
        is our chosen unit."""
        u1amt = amt1 + amt2 * (1 / float(conv))
#        print u1amt, u1,
        u2amt = amt2 + amt1 * conv
#        print 'or', u2amt, u2
#        print 'u1:', self.readability_score(u1amt)
#        print 'u2:', self.readability_score(u2amt)
        if self.readability_score(u1amt) >= self.readability_score(u2amt):
            return [u1amt, u1]
        else:
            return [u2amt, u2]

    def add_reasonably (self, a1, u1, a2, u2, item=None):
        """Return a list with amount and unit if conversion is possible.
        Else return None"""
        if not (a1 and a2):
            # we give up if these aren't numbers
            return None
        conv = self.converter(u1,u2, item)
        if conv:
            ## Don't bother with all the work if the conversion factor is 1
            if conv == 1:
                return [a1 + a2, u1]
            else:
                return self.use_reasonable_unit(a1, u1, a2, u2, conv)
        else:
            return None

    def amt_string(self, amt, approx=0.01):
        """Given list of [amount unit], hand back a string
        representing amount.  We'll do our best to turn numbers back
        into fractions here so that they will look easily readable"""
        num = amt[0]
        un = amt[1]
        return "%s %s" %(float_to_frac(num, approx=approx), un)

    
def integerp (num, approx=0):
    """approx can be a decimal that is a guide to rounding.
    That is, if approx is 0.001, then we will consider
    0.9991 and 1.0009 both to be the integer 1.  This feature
    will only work for positive, non-zero numbers."""
    try:
        if int(num) == float(num):
            return int(num)
        elif approx:
            bigside = int(num+approx)
            smallside = int(num-approx)
            if bigside != smallside:
                return bigside
            else:
                return None
    except:
        print "Warning: big big numbers..."
        return None


def float_to_frac (n, d=[2,3,4,5,6,8,10,16],approx=0.01):
    """Take a number -- or anything that can become a float --
    and attempt to return a fraction with a denominator in the list `d'. We
    approximate fractions to within approx. i.e. if approx=0.01, then 0.331=1/3"""
    if not n: return ""
    n=float(n)
    i = int(n)
    if i >= 1:
        i="%s "%int(n)
    else:
        i=""
    rem = n - int(n)
    if rem==0:
        if i:
            return "%s"%i
        else:
            return "0"
    else:
        def fractify(decimal, divisor, approx):
            dividend = decimal*divisor
            if integerp(dividend, approx):
                return "%s/%s"%(integerp(dividend, approx), divisor)
            else:
                return False
        flag = False
        for div in d:
            f = fractify(rem,div,approx)
            if f:
                flag = True
                return "%s%s"%(i,f)
        if not flag:
            if len("%s"%n) > 3:
                return "%.2f"%(n)
            else:
                return "%s"%n
        

def frac_to_float (s):
    """We assume fractions look like this (I )?N/D"""
    if s.find("/"):
        m=re.match("([0-9]+\s+)?([0-9]+)/([1-9]+)",s)
        if m:
            i,n,d=m.groups()
            if not i: i=0
            return float(i)+(float(n)/float(d))
        else:
            try:
                return float(s)
            except:
                None
    else:
        #If this isn't a fraction, we're just using float
        try:
            return float(s)
        except:
            None



if __name__ == '__main__':
    c = converter()
    print "Testing..."
    print 'g. to l. water', c.converter('g.','l.',item="water")
    print 'testing our readability measures'
    print '9 tsp. + 4 tbs.', c.add_reasonably(9,'tsp.',4,'tbs.')
    print '3 cups + 3 tbs. water', c.add_reasonably(3, 'c.', 3, 'tbs.', 'water')
