#!/usr/bin/env python
import gtk, os.path, optionTable

H_PADDING=12
Y_PADDING=12

class mDialog (gtk.Dialog):
    def __init__ (self, default=None, okay=True, label=False, parent=None, cancel=True, modal=True, label_markup=True, expander=None):
        """Out basic class. We allow for a label. Possibly an expander
        with extra information, and a simple Okay button.  The
        expander is are only fancy option. It should be a list ['Name
        of expander', CONTENTS]. CONTENTS can be a string (to be put
        in a label), a widget (to be packed in the expander), or a
        list of strings and widgets to be packed in order."""
        gtk.Dialog.__init__(self, parent=parent)
        self.connect('destroy',self.cancelcb)
        self.set_border_width(15)
        self.default = default
        self.ret = default
        if modal: self.set_modal(gtk.TRUE)
        if label:
            self.label = gtk.Label(label)
            self.vbox.pack_start(self.label,expand=False)
            self.label.set_padding(H_PADDING,Y_PADDING)
            self.label.set_use_markup(label_markup)
            self.label.show()
        if expander:
            label=expander[0]
            body = expander[1]
            self.expander = gtk.Expander(label)
            self.expander_vbox = gtk.VBox()
            self.expander.add(self.expander_vbox)
            self._add_expander_item(body)
            self.expander.show()
            self.expander_vbox.show_all()
            self.vbox.add(self.expander)
        if cancel:
            self.cancel = gtk.Button(stock=gtk.STOCK_CANCEL)
            self.action_area.add(self.cancel)
            self.cancel.show()
            self.cancel.connect('clicked',self.cancelcb)
        if okay:
            self.ok = gtk.Button(stock=gtk.STOCK_OK)
            self.action_area.add(self.ok)        
            self.ok.show()
            self.ok.connect('clicked',self.okcb)
            self.ok.grab_focus()
            
    def _add_expander_item (self, item):
        if type(item)==type(""):
            self.expander_vbox.pack_start(Label(item),
                                          expand=False,
                                          fill=False)
        elif type(item)==[] or type(item)==():
            map(self._add_expander_item,item)
        else:
            self.expander_vbox.pack_start(item)
            
    def run (self):
        self.show()
        if self.modal: gtk.mainloop()
        return self.ret

    def okcb (self, *args):
        self.hide()
        if self.modal: gtk.mainquit()

    def cancelcb (self, *args):
        self.hide()
        self.ret=None
        if self.modal: gtk.mainquit()

class numberDialog (mDialog):
    def __init__(self,default=None,label=False,step_incr=1,page_incr=10,digits=0,
                 min=0,max=10000, parent=None):
        mDialog.__init__(self,default=default, parent=parent)
        self.hbox=gtk.HBox()
        self.vbox.add(self.hbox)
        #self.spinButton=gtk.SpinButton(climb_rate=climb_rate,digits=digits)
        if not default:
            val = 0
        else:
            val = float(default)
        self.adjustment=gtk.Adjustment(val,
                                       lower=min,
                                       upper=max,
                                       step_incr=step_incr,
                                       page_incr=page_incr)
        self.spinButton=gtk.SpinButton(self.adjustment)
        if default:
            self.spinButton.set_value(default)
        if label:
            self.label=gtk.Label(label)
            self.label.set_padding(H_PADDING, Y_PADDING)
            self.hbox.add(self.label)
            self.label.show()
        self.hbox.add(self.spinButton)
        self.spinButton.get_adjustment().connect("value_changed",self.update_value)
        self.spinButton.show()
        self.hbox.show()                
        
    def update_value (self, *args):
        self.ret=self.spinButton.get_value()

class entryDialog (mDialog):
    def __init__ (self, default=None, label=False, parent=None):
        mDialog.__init__(self,default=default,parent=parent)
        self.hbox=gtk.HBox()
        self.vbox.add(self.hbox)        
        if label:
            self.label=gtk.Label(label)
            self.hbox.add(self.label)
            self.label.show()
            self.label.set_padding(H_PADDING,Y_PADDING)
        self.entry = gtk.Entry()
        self.hbox.add(self.entry)
        self.hbox.show()
        if default:
            self.entry.set_text(default)
        self.entry.connect("changed",self.update_value)
        self.entry.show()

    def update_value (self, *args):
        self.ret = self.entry.get_text()

class optionDialog (mDialog):
    def __init__ (self, default=None, label="Select Option",options=[], parent=None):
        """Options can be a simple option or can be a tuple or a list
        where the first item is the label and the second the value"""
        mDialog.__init__(self, okay=True, label=label, parent=parent)
        self.optdic={}
        self.menu = gtk.Menu()
        for o in options:
            if type(o)==type(""):
                l=o
                v=o
            else:
                l=o[0]
                v=o[1]
            i = gtk.MenuItem(l)
            i.connect('activate',self.menucb)
            i.show()
            self.optdic[i]=v
            self.menu.append(i)
        print "DEBUG: %s"%self.optdic
        self.optionMenu=gtk.OptionMenu()
        self.vbox.add(self.optionMenu)
        self.optionMenu.set_menu(self.menu)
        self.optionMenu.show()
        self.menu.show()

    def menucb (self, widget):
        self.ret=self.optdic[widget]
        print "DEBUG: Set value to ",self.ret

    def set_value (self, value):
        print "Setting value ",value
        self.ret=value
        
class preferences_dialog (mDialog):
    def __init__ (self, options=([None,None]), option_label="Option",
                  value_label="Value", default=True, label=None,
                  apply_func=None, parent=None):
        """Options is a tuple of lists where each list is ['option', VALUE], handed to OptionTable
        
        VALUE can be any of the following:
        a string (will be editable)
        a number (will be editable and returned as a number)
        true or false (will be a checkbox)

        If apply_func is True, we will have an apply button, which
        will hand the option tuple as its argument. Otherwise, okay will simply
        return the list on okay."""
        if apply_func: modal=False
        else: modal=True
        mDialog.__init__(self, okay=True, label=label, parent=parent, modal=modal)
        self.apply_func = apply_func
        self.options = options
        self.revert = gtk.Button(stock=gtk.STOCK_REVERT_TO_SAVED)
        self.revert.connect('clicked',self.revertcb)
        if self.apply_func:
            self.action_area.add(self.revert)
            self.apply = gtk.Button(stock=gtk.STOCK_APPLY)
            self.apply.set_sensitive(gtk.FALSE)
            self.apply.connect('clicked',self.applycb)
            self.action_area.add(self.apply)
            self.apply.show()
            changedcb = lambda *args: self.apply.set_sensitive(gtk.TRUE)
        else:
            changedcb=None
            self.set_modal(gtk.FALSE)
        self.ok = gtk.Button(stock=gtk.STOCK_OK)
        self.ok.connect('clicked',self.okcb)
        self.action_area.add(self.ok)
        self.table = optionTable.OptionTable(options=self.options,
                                             option_label=option_label,
                                             value_label=value_label,
                                             changedcb=changedcb)
        self.vbox.add(self.table)
        self.table.show()
        
    def revertcb (self, *args):
        self.table.revert()
        
    def applycb (self, *args):
        self.table.apply()
        self.apply_func(self.table.options)
        self.apply.set_sensitive(gtk.FALSE)

    def run (self):
        self.show()
        if self.apply_func:
            return
        else:
            gtk.mainloop()
            return self.ret
        
    def okcb (self, *args):
        if self.apply_func:
            #print 'ok: apply_func mode'
            if self.apply.get_property('sensitive'):
                # if there are unsaved changes...
                if getBoolean(label="Would you like to apply the changes you've made?"):
                    self.applycb()
            self.hide()
        else:
            #print 'ok: modal mode'
            self.table.apply()
            self.ret = self.table.options
            self.hide()
            gtk.main_quit()
                   
    def cancelcb (self, value):
        self.hide()
        self.ret=None
        
        
class booleanDialog (mDialog):
    def __init__ (self, default=True, label="Do you really want to do this",
                  parent=None, custom_yes=None, custom_no=None, expander=None):
        mDialog.__init__(self,okay=False,label=label,parent=parent, expander=expander)
        if custom_no: self.no = gtk.Button(custom_no)
        else: self.no = gtk.Button(stock=gtk.STOCK_NO)
        if custom_yes: self.yes = gtk.Button(custom_yes)
        else: self.yes = gtk.Button(stock=gtk.STOCK_YES)
        self.action_area.add(self.no)
        self.action_area.add(self.yes)
        self.yes.connect('clicked',self.yescb)
        self.no.connect('clicked',self.nocb)
        self.yes.show()
        self.no.show()
        
    def yescb (self, *args):
        self.ret=True
        self.okcb()

    def nocb (self, *args):
        self.ret=False
        self.okcb()


class FileSelection (gtk.FileSelection):
    def __init__ (self, retFun, winTitle="Select File", filename=None, file_required=1):
        """retFun will be handed filename when done"""
        self.file_required = file_required
        self.rf = retFun
        gtk.FileSelection.__init__(self,winTitle)
        if filename:
            self.set_filename(filename)
        self.ok_button.connect('clicked',self.ok)
        self.cancel_button.connect('clicked',self.quit)
        self.fn = self.get_filename()
        self.run()
        
    def quit (self, *args):
        self.hide()
        
    def ok (self, *args):
        print 'DEBUG: OK button pressed'
        self.fn = self.get_filename()
        self.fn = os.path.normpath(self.fn)
        if self.file_required and os.path.isdir(self.fn):
            print "This is a directory!"
            if self.fn[-1] != '/': self.fn += '/'
            self.set_filename(self.fn)
        else:
            print "Calling %s on file %s"%(self.rf,self.fn)
            self.hide()
            self.rf(self.fn)
            self.destroy()

        
class FileSelectionM (gtk.FileSelection):
    def __init__ (self, winTitle="Select File", filename=None, file_required=1):
        self.file_required = file_required
        gtk.FileSelection.__init__(self,winTitle)
        self.set_modal(gtk.TRUE)
        if filename:
            self.set_filename(filename)
        self.ok_button.connect('clicked',self.ok)
        self.cancel_button.connect('clicked',self.quit)
        self.fn = self.get_filename()
        self.retval=None
        
    def run(self):
        self.show()
        gtk.mainloop()
        return self.retval
        
    def quit (self, *args):
        self.retval=None
        self.hide()
        gtk.mainquit()
        #threading.
        
    def ok (self, *args):
        print 'DEBUG: OK button pressed'
        self.fn = self.get_filename()
        self.fn = os.path.normpath(self.fn)
        if self.file_required and os.path.isdir(self.fn):
            print "This is a directory!"
            if self.fn[-1] != '/': self.fn += '/'
            self.set_filename(self.fn)
        else:
            self.hide()
            self.retval=self.get_filename()
            gtk.mainquit()


def show_message (*args, **kwargs):
    d=mDialog(*args,**kwargs)
    d.run()
    return d

def select_file (*args, **kwargs):
    """Run FileSelectionM dialog, passing along all args
    and waiting to return the result."""
    fsd = FileSelectionM(*args, **kwargs)
    return fsd.run()

def getNumber (*args, **kwargs):
    """Run numberDialog, passing along all args, waiting on input and passing along
    the results."""
    d = numberDialog(*args, **kwargs)
    return d.run()
                     
def getEntry (*args, **kwargs):
    """Run entryDialog, passing along all args, waiting on input and passing along
    the results."""    
    d = entryDialog(*args, **kwargs)
    return d.run()

def getBoolean (*args,**kwargs):
    """Run booleanDialog, passing along all args, waiting on input and
    passing along the results."""
    d = booleanDialog(*args,**kwargs)
    retval = d.run()
    if retval==None:
        raise "getBoolean dialog cancelled!"
    else:
        return retval

def getOption (*args,**kwargs):
    d=optionDialog(*args,**kwargs)
    return d.run()

if __name__ == '__main__':
    w=gtk.Window()
    w.connect('delete_event',gtk.main_quit)
    b=gtk.Button("show dialog (modal)")
    opts=(["Toggle Option",True],
          ["String Option","Hello"],
          ["Integer Option",1],
          ["Float Option",float(3)])
    pd=preferences_dialog(options=opts)
    def run_prefs (*args):
        print pd.run()
    b.connect('clicked',run_prefs)
    def show_options (options):
        print options
    b2=gtk.Button("show dialog (not modal)")
    b2.connect('clicked',lambda *args: preferences_dialog(options=opts,apply_func=show_options).show())
    vb=gtk.VBox()
    w.add(vb)
    vb.add(b)
    b.show()
    vb.add(b2)
    b2.show()
    vb.show()
    w.show()
    gtk.main()
