#!/usr/bin/python

import rmetakit, importer, re, convert, os.path
from debug import debug

class mmf_constants:
    def __init__ (self):
        self.recattrs={'Title':'title',
                       'Categories':'category',
                       'Servings':'servings',
                       'Source':'source',
                       'Recipe by':'source',
                       'Yield':'servings',
                       'Preparation Time':'preptime',
                       }
        self.unit_conv = {'ts':'tsp.',
                          'tb':'tbs.',
                          'sm':'small',
                          'md':'medium',
                          'lg':'large',
                          'c':'c.',
                          'pn':'pinch',
                          'ds':'dash',
                          'T' : 'tbs.',
                          'pk' : 'package',
                          'x' : '',
                          't' : 'tsp.',
                          }
        self.unit_convr = {}
        for k,v in self.unit_conv.items():
            self.unit_convr[v]=k

        

mmf=mmf_constants()
class mmf_importer (importer.importer):
    def __init__ (self,rd,filename='Data/mealmaster.mmf',
                  progress=None, source=None,threaded=True):
        debug("start  __init__ ",5)
        """filename is the file to parse. rd is the recData instance
        to start with.  progress is a function we tell about our
        progress to (we hand it a single arg)."""
        importer.importer.__init__(self,rd=rd,threaded=threaded)
        self.rec={}
        self.source=source
        self.instrp=False
        self.header=False
        self.instr=""
        self.ingrp=False
        self.attrp=False
        self.ingrs=[]
        self.ing_added=False
        self.compile_regexps()        
        self.file=open(filename,'r')
        self.base=os.path.split(filename)[1]
        ll=self.file.readlines() #slurp
        tot=len(ll)
        for n in range(tot):
            # we do the loop this way so we can
            # conveniently report our progress to
            # the outside world :)
            l=ll[n]
            if float(n)/10==n/10:
                prog= float(n)/float(tot)
                #print "DEBUG: progress=",prog
                if progress:
                    progress(prog)

            self.handle_line(l)
        if self.rec:
            self._commit_rec()

    def compile_regexps (self):
        debug("start compile_regexps",5)
        self.start_matcher = re.compile("^MMMMM-*\s*Recipe.*")
        self.end_matcher = re.compile("^MMMMM\s*$")
        self.group_matcher = re.compile("^MMMMM-+\s*([^-]+)\s*-*")
        self.blank_matcher = re.compile("^\s*$")
        self.ing_cont_matcher = re.compile("^\s*[-;]")
        self.ing_opt_matcher = re.compile("(.+)\s*\(?\s*[Oo]ptional\)?\s*$")
        self.ing_or_matcher = re.compile("^[- ]*[Oo][Rr][- ]*$")
        # we build a regexp to match anything that looks like
        # this: ^\s*ATTRIBUTE: Some entry of some kind...$
        attrmatch="^\s*("
        for k in mmf.recattrs.keys():
            attrmatch += "%s|"%re.escape(k)
        attrmatch="%s):\s*(.*)\s*$"%attrmatch[0:-1]
        self.attr_matcher = re.compile(attrmatch)
        
    def handle_line(self,l):
        debug("start handle_line",5)
        if self.start_matcher.match(l):
            #print "DEBUG: New recipe"
            debug("recipe start %s"%l,6)
            self.new_rec()
            return
        if self.end_matcher.match(l):
            debug("recipe end %s"%l,6)            
            self._commit_rec()
            return
        groupm = self.group_matcher.match(l)
        if groupm:
            debug("new group %s"%l,6)            
            self.handle_group(groupm)                    
            return
        attrm = self.attr_matcher.match(l)
        if attrm:        
            # a match for an attribute has two groups,
            # (ATTRIBUTE): (VALUE)
            attr,val = attrm.groups()
            debug("attribute %s (%s:%s)"%(l,attr,val),6)            
            self.rec[mmf.recattrs[attr]]=val
            self.attrp = True
            return

        if self.blank_matcher.match(l):
            debug("blank %s"%l,6)            
            # a blank starts our ingredient list
            if self.attrp and not self.ingrp and not self.instrp:
                debug("blank line starts ingredients",6)
                self.ingrp=True
                return
            # or it ends our ingredient list
            if self.ingrp:
                debug("blank line starts instructions",6)
                self.ingrp=False
                self.instrp=True
                return
            if not self.instrp:
                debug("blank line ignored",6)
                return
            # otherwise, we're in instructions
            # so the blank line should be added
            # as part of the instruction text!
        
        if self.instrp:
            debug("instruction %s"%l,6)
            # once we're in instructions, we just keep on adding
            # until we're done.
            self.instr = "%s%s"%(self.instr,l)
            return
        
        if self.ingrp:
            # we handle continuations of long lines
            debug("ingredients %s"%l,6)            
            contm=self.ing_cont_matcher.match(l)
            if contm and self.ingrs:
                # the end of the match will start the continued text
                debug('continuing %s'%self.ingrs[-1][0],3)
                continuation = " %s"%(l[contm.end():].strip())
                self.ingrs[-1][0] += continuation
            else:
                self.ingrs.append([l,self.current_group])

        else:
            debug("unhandled %s"%l,6)
            
    def new_rec (self):
        debug("start new_rec",5)
        if self.rec:
            # this shouldn't happen if recipes are ended properly
            # but we'll be graceful if a recipe starts before another
            # has ended... 
            self._commit_rec
        self.start_rec(base=self.base)
        self.instrp=False
        self.instr=""
        self.ingrp=False
        self.attrp=False
        self.ingrs=[]
        self.current_group=None
        self.header=False

    def _commit_rec (self):
        debug("start _commit_rec",5)
        self.rec['instructions']=self.instr
        self.parse_inglist()
        if self.source:
            self.rec['source']=self.source
        self.commit_rec()

    def handle_group (self, groupm):
        debug("start handle_group",5)
        # the only group of the match will contain
        # the name of the group. We'll put it into
        # a more sane title case (MealMaster defaults
        # to all caps
        name = groupm.groups()[0].title()
        self.current_group=name
        # a blank line before a group could fool us into thinking
        # we were in instructions. If we see a group heading,
        # we know that's not the case!
        self.ingrp = True 
        self.instrp = False        

    def find_ing_fields (self):
        debug("start find_ing_fields",5)
        all_ings = map(lambda tupl: tupl[0], self.ingrs)
        fields = find_fields(all_ings)        
        a = []
        while fields and field_match(all_ings,fields[0],
                                     "^[0-9- /]+$"):
            # amounts are 
            a.append(fields[0])
            del fields[0]
        if a:
            a=(a[0][0],a[-1][1]) #a is the range from least to most
        if fields and field_match(all_ings,fields[0],
                                  "^..?$"):
            u=fields[0]
            del fields[0]
        else: u=""
        if fields:
            i=(fields[0][0],fields[-1][1])
        else:
            debug("No items? this seems odd.",0)
            i=""
        debug("Returning fields: %s,%s,%s"%(a,u,i),4)
        return a,u,i
            
                
    def parse_inglist(self):
        debug("start parse_inglist",5)
        """We handle our ingredients after the fact."""
        afield,ufield,ifield=self.find_ing_fields()
        debug("ingredient fields are: %s,%s,%s"%(afield,ufield,ifield),6)
        for s,g in self.ingrs:
            self.start_ing()
            self.group = g
            amt,u,i = get_fields(s,(afield,ufield,ifield))
            if amt:
                self.add_amt(amt)
            if u:
                self.add_unit(u)
            optm=self.ing_opt_matcher.match(i)
            if optm:
                item=optm.groups()[0]
                self.add_item(item)
                self.ing['optional']='yes'
            #elif self.ing_or_matcher.match(i):
            #    ?
            else:
                self.add_item(i)
            debug("committing ing: %s"%self.ing,6)
            self.commit_ing()
                

def split_fields (strings, char=" "):
    debug("start split_fields",5)
    fields=find_fields(strings,char)

def field_match (strings, tuple, matcher):
    debug("start field_match",5)
    if type(matcher)==type(""):
        matcher=re.compile(matcher)
    for s in strings:
        f=s[tuple[0]:tuple[1]]
        if not matcher.match(f):
            return False
    return True

def get_fields (string, tuples):
    debug("start get_fields",5)
    lst = []
    for t in tuples:
        if t:
            lst.append(string[t[0]:t[1]])
        else:
            lst.append("")
    return lst

def field_width (tuple):
    debug("start field_width",5)
    if tuple[1]:
        return tuple[1]-tuple[0]
    else:
        return None

def find_fields (strings, char=" "):
    debug("start find_fields",5)
    cols=find_columns(strings, char)
    if not cols:
        return
    fields = []
    if cols[0] > 0:
        fields.append((0,cols[0]))
    range_start = None
    for n in range(cols[0],cols[-1]+1):
        if n not in cols:
            if not range_start: range_start = n
        else:
            if range_start:
                fields.append((range_start,n))
                range_start=None
    if range_start: fields.append((range_start,None))
    else: fields.append((cols[-1]+1,None))
    return fields
            
def find_columns (strings, char=" "):
    debug("start find_columns",5)
    # we start with the columns in the first string
    if not strings:
        return None
    columns = map(lambda match: match.start(),re.finditer(re.escape(char),strings[0]))
    if len(strings)==1:
        return columns
    # we eliminate all columns that aren't blank for every string
    for s in strings:
        for c in columns[0:]: # we'll be modifying columns
            if c < len(s) and s[c]!=char:
                columns.remove(c)
    columns.sort()
    return columns
        
if __name__ == '__main__':
    def printprog (prog):
        print "%i percent done"%(prog * 100)
    mmf_importer(rmetakit.recipeManager(),'/home/tom/Projects/MM132501',
                 progress=printprog, source="MM132501")
    
    
