#!/usr/bin/env python

import gtk

class TimeEntry (gtk.HBox):
    def __init__ (self, *args, **kwargs):
        gtk.HBox.__init__(self, *args, **kwargs)
        self.adj = gtk.Adjustment(value=0, lower=0, upper=100, step_incr=1, page_incr=10)
        self.spin = gtk.SpinButton(adjustment=self.adj, digits=1)
        self.spin.set_numeric(gtk.TRUE)
        self.spin.set_update_policy(gtk.UPDATE_IF_VALID)
        self.combo = gtk.Combo()
        self.units = ['seconds','minutes','hours','days','weeks','months']
        self.combo.set_popdown_strings(self.units)
        self.pack_start(self.spin, gtk.TRUE, gtk.TRUE, 6)
        self.spin.show()
        self.pack_start(self.combo, gtk.TRUE, gtk.TRUE, 6)
        self.combo.show()
        self.combo.entry.set_text('minutes')
        
    def set_text (self, text):
        """We only accept text of the form '# unit'"""
        try:
            num,unit = text.split()
        except:
            num,unit = 0,"minutes" #defaults
        self.spin.set_value(float(num))
        self.combo.entry.set_text(str(unit))
        
    def get_text (self):
        """We return text in the form of '# unit'"""
        num,unit=self.get_value()
        if int(num)==num: num=int(num)
        return "%s %s"%(num,unit)

    def get_value (self):
        """We return a tuple #,unit"""
        return (self.spin.get_value(),self.combo.entry.get_text())

    def set_value (self, number, unit):
        self.spin.set_value(float(number))
        self.combo.entry.set_text(str(unit))

    def connect (self, signal, handler):
        self.spin.connect(signal, handler)
        self.combo.entry.connect(signal, handler)
        
def makeTimeEntry ():
    te=TimeEntry()
    te.show()
    return te

if __name__ == '__main__':
    w=gtk.Window()
    te=TimeEntry()
    w.add(te)
    te.show()
    w.show()
    w.connect('delete_event',gtk.main_quit)
    gtk.main()
    
